//  JAWTAtlantis.java -- JAWTAtlantis
//
//  04/14/06 Oleg Kobchenko - Created.
//  Copyright 2006 Oleg Kobchenko. All rights reserved.
//

package olegyk.jawtopengl.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.Timer;
import javax.swing.*;
import java.util.*;
import olegyk.jawtopengl.*;

public class JAWTAtlantis extends JFrame
{
	static int offset = 0;
	Container pane = getContentPane();
	
    public JAWTAtlantis() {
        super("JAWTAtlantis");
		// removes frame and children, exits after last window closed
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
		offset = (offset + 30) % 600;
		setLocation(offset, offset);
        setSize(500, 500);
    }
    
    public void addNotify() {
        super.addNotify();

        final OpenGLCanvas canvas = new OpenGLCanvas() {
			public void init() {
				Atlantis.init(); 
			}
			public void reshape(int width, int height) {
				Atlantis.reshape(width, height); 
			}
			public void paint() {
				Atlantis.display();
				Atlantis.animate();
			}
		};
		final Label fpsLabel = new Label("0 fps");
        pane.setLayout(new BorderLayout(2, 2));
		pane.add(fpsLabel, BorderLayout.NORTH);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
        pane.add(canvas, BorderLayout.CENTER);
		pane.add(new Label("  Fast, lightweight, native OpenGL"), BorderLayout.SOUTH);
		
		final Timer timer = new Timer(10, new ActionListener() {
			static final int freq = 100;
			long count = 0, time = System.currentTimeMillis();

			public void actionPerformed(ActionEvent evt) {
				canvas.repaint();
				if (0 == ++count % freq) {
					long now = System.currentTimeMillis();
					fpsLabel.setText(freq*1000/(now-time) + " fps");
					time = now;
				}
			}
		});
		canvas.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (timer.isRunning()) timer.stop(); else timer.start();
			}
		});
		addWindowListener(new WindowAdapter() {
			// attempt to paint canvas before it became
			// first visible will cause "invalid drawable"
			// during [OpenGLContext setView:]
			public void windowOpened(WindowEvent e) {
				timer.start();
			}
			public void windowClosing(WindowEvent e) {
				if (timer.isRunning()) timer.stop();
			}
		});
	}
	    
    public static void main(String args[]) {
		new JAWTAtlantis().show();
    }    
}
