//  OpenGLCanvas.c -- JAWTOpenGL
//
//  04/14/06 Oleg Kobchenko - Created
//  04/25/06 Oleg Kobchenko - Win32
//  05/02/06 Oleg Kobchenko - X11
//  05/04/06 Oleg Kobchenko - better release
//  Copyright 2006 Oleg Kobchenko. All rights reserved.
//

#include <X11/Xlib.h>
#include <GL/glx.h>
#include <GL/gl.h>
#include <assert.h>
#include <jawt_md.h>

#include <olegyk_jawtopengl_OpenGLCanvas.h>

#define JAVA_EXCEPT()						\
    if ((*env)->ExceptionOccurred(env)) {	\
        (*env)->ExceptionDescribe(env);		\
    }


static int	sngBuf[] = {	GLX_RGBA,
				GLX_RED_SIZE, 1,
				GLX_GREEN_SIZE, 1,
				GLX_BLUE_SIZE, 1,
				GLX_DEPTH_SIZE, 12,
				None };

static int	dblBuf[] = {	GLX_RGBA,
				GLX_RED_SIZE, 1,
				GLX_GREEN_SIZE, 1,
				GLX_BLUE_SIZE, 1,
				GLX_DEPTH_SIZE, 12,
				GLX_DOUBLEBUFFER,
				None };

Bool doubleBuffer = True;

Colormap defaultPixelFormat(Display *dpy, XVisualInfo *vi) {
	return XCreateColormap(dpy,RootWindow(dpy,vi->screen),vi->visual,AllocNone);
}

GLXContext ensureContext(JAWT_X11DrawingSurfaceInfo* dsi_x11, GLXContext cx) 
{
	Display* dpy = dsi_x11->display;
	if (!cx) {
		int dummy = 0;
		XVisualInfo *vi = NULL;

		if (!glXQueryExtension(dpy, &dummy, &dummy))
    			return 0; // X server has no OpenGL GLX extension
                                                                                
		if (!(vi = glXChooseVisual(dpy, DefaultScreen(dpy), dblBuf))) {
   			if(!(vi = glXChooseVisual(dpy, DefaultScreen(dpy), sngBuf)))
				return 0; // no RGB visual with depth buffer
			doubleBuffer = False;
		}

		if (vi->class != TrueColor)
			return 0; // TrueColor visual required for this program
                                                                                
		if (!(cx = glXCreateContext(dpy, vi,
                		None,   /* no sharing of display lists */
		                True    /* direct rendering if possible */
	                )))
			return 0; // could not create rendering context
                                                                                

		// Colormap cmap = defaultPixelFormat(dsi_x11->display, vi);
	}

	if (glXGetCurrentDrawable() != dsi_x11->drawable) {
		assert(glXMakeCurrent(dpy, dsi_x11->drawable, cx));
	}
	
	return cx;
}

typedef struct {
	JAWT* awt;
	JAWT_DrawingSurface* ds;
	JAWT_DrawingSurfaceInfo* dsi;
	JAWT_X11DrawingSurfaceInfo* dsi_x11;
	GLXContext cx;
} ContextInfo;

static jfieldID ctxID = NULL;

ContextInfo* getContext(JNIEnv *env, jobject canvas) 
{
	ContextInfo *ci;
	if (!ctxID) {
		jclass cls = (*env)->GetObjectClass(env, canvas);
		ctxID = (*env)->GetFieldID(env, cls, "openGLContext", "J");
	}	

	ci = (ContextInfo *)(long)((*env)->GetLongField(env, canvas, ctxID));

	if (!ci) {
		ci = (ContextInfo *)calloc(sizeof(ContextInfo), 1);
		ci->awt = (JAWT *)calloc(sizeof(JAWT), 1);
		(*env)->SetLongField(env, canvas, ctxID, (jlong)(long)ci);
	}
	
	return ci;
}

void freeContext(JNIEnv *env, jobject canvas, ContextInfo* ci) 
{
	if (ci) {
		free(ci->awt);
		free(ci);
		(*env)->SetLongField(env, canvas, ctxID, 0L);
	}
}


JNIEXPORT jboolean JNICALL Java_olegyk_jawtopengl_OpenGLCanvas_beginOpenGL(
			  JNIEnv *env, jobject canvas)
{
	jint lock;
	ContextInfo *ci = getContext(env, canvas);
	
	// Lock the drawing surface
    // You must lock EACH TIME before drawing
    lock = ci->ds->Lock(ci->ds); 
	JAVA_EXCEPT();

    assert((lock & JAWT_LOCK_ERROR) == 0);
	if (lock & JAWT_LOCK_ERROR) {
		return JNI_FALSE;
	}
    
    // Get the drawing surface info
    ci->dsi = ci->ds->GetDrawingSurfaceInfo(ci->ds);
	
	// Check DrawingSurfaceInfo.  This can be NULL
    // if the windowing system is not ready
	if (ci->dsi != NULL) {
        // Get the platform-specific drawing info
        ci->dsi_x11 = (JAWT_X11DrawingSurfaceInfo*)ci->dsi->platformInfo;
		JAVA_EXCEPT();

        // Get the corresponding peer from the caller canvas
		ci->cx = ensureContext(ci->dsi_x11, ci->cx);

		return ci->cx != NULL;
	}
	
	return JNI_FALSE;
}

JNIEXPORT void JNICALL Java_olegyk_jawtopengl_OpenGLCanvas_endOpenGL(
				JNIEnv *env, jobject canvas)
{
	ContextInfo *ci = getContext(env, canvas);
	
	if(doubleBuffer) {
		glXSwapBuffers(ci->dsi_x11->display,ci->dsi_x11->drawable);  // buffer swap does implicit glFlush
	} else {
		glFlush();			// explicit flush for single buf case
	}

	// Free the DrawingSurfaceInfo
	ci->ds->FreeDrawingSurfaceInfo(ci->dsi);
	JAVA_EXCEPT();
	
    // Unlock the drawing surface
    // You must unlock EACH TIME when done drawing
    ci->ds->Unlock(ci->ds); 
	JAVA_EXCEPT();

}

JNIEXPORT void JNICALL Java_olegyk_jawtopengl_OpenGLCanvas_updateOpenGL(
			   JNIEnv *env, jobject canvas)
{
	ContextInfo *ci = getContext(env, canvas);

//	wglMakeCurrent( ci->dsi_win->hdc, ci->hRC );
}

JNIEXPORT void JNICALL Java_olegyk_jawtopengl_OpenGLCanvas_allocOpenGL(
			JNIEnv *env, jobject canvas)
{
	ContextInfo *ci = getContext(env, canvas);
	
	jboolean result = JNI_FALSE;
    
    // get the AWT
    ci->awt->version = JAWT_VERSION_1_4;
    result = JAWT_GetAWT(env, ci->awt);
	JAVA_EXCEPT();
    assert(result != JNI_FALSE);	

    // Get the drawing surface.  This can be safely cached -- not in win32
    // Anything below the DS (DSI, contexts, etc) 
    // can possibly change/go away and should not be cached.
    ci->ds = ci->awt->GetDrawingSurface(env, canvas);
	JAVA_EXCEPT();
    assert(ci->ds != NULL);

}

JNIEXPORT void JNICALL Java_olegyk_jawtopengl_OpenGLCanvas_releaseOpenGL(
			JNIEnv *env, jobject canvas) 
{
	ContextInfo *ci = getContext(env, canvas);

	// required to avoid "Xlib: unexpected async reply"
	Java_olegyk_jawtopengl_OpenGLCanvas_beginOpenGL(env, canvas);

	if (ci->cx) {
		glXMakeCurrent(ci->dsi_x11->display, None, NULL );
		glXDestroyContext(ci->dsi_x11->display, ci->cx);
	}

	Java_olegyk_jawtopengl_OpenGLCanvas_endOpenGL(env, canvas);

    // Free the drawing surface (if not caching it)
    ci->awt->FreeDrawingSurface(ci->ds);
	JAVA_EXCEPT();
		
	freeContext(env, canvas, ci);
}
